from __future__ import unicode_literals

from mock import patch

from django.test import TestCase
from django.core.urlresolvers import reverse

from eVotUM.Cripto import pkiutils

from evotevotings.elections.utils import get_extra_election_data

from evotevotings.admin.utils import (get_certificates,
                                      not_expired_certificate)
from evotevotings.services.validator.tests.utils import (
    ValidatorServiceTestUtils)

from . import ServiceTestUtils, ValidCertificateTestCase
from evotevotings.services import jssettings


class APIViewTest(TestCase):
    """ """
    def setUp(self):
        self.url = reverse('genericservice:swagger')

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'rest_framework_swagger/index.html')


class JavascriptSettingsTest(TestCase):
    """ """
    def setUp(self):
        self.jss = jssettings.javascript_settings()
        self.vvote_endpoint = self.jss['endpoints']['validateVote']
        self.fsig_endpoint = self.jss['endpoints']['filterSignature']

    def test_data_structure(self):
        self.assertIn('endpoints', self.jss)

    def test_filtersignature_url(self):
        validate_vote = reverse('genericservice:filtersignature')
        self.assertIn('filterSignature', self.jss['endpoints'])
        self.assertEquals(self.fsig_endpoint, validate_vote)

    def test_validatevote_url(self):
        validate_vote = reverse('genericservice:validatevote')
        self.assertIn('validateVote', self.jss['endpoints'])
        self.assertEquals(self.vvote_endpoint, validate_vote)


class StepZeroTest(ValidCertificateTestCase, ServiceTestUtils):
    """ Class to test the services workflow Step 0"""

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    # Test Units
    def test_dataintegrity(self):
        """ Check if election data was properly loaded """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.user_id)
        self.assertTrue(extra_data)

    def test_invalid_election_id(self):
        """ Check unexpected election id """
        extra_data = get_extra_election_data(-1,
                                             self.voter.user_id)
        self.assertFalse(extra_data)

    @patch("evotevotings.elections.utils.get_secret_entry", return_value=False)
    def test_invalid_certificate(self, mock_fn):
        """ Check for invalid certificates """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.user_id)
        self.assertFalse(extra_data)

    def test_valid_certificate(self):
        """ Check if election certificate is valid """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.user_id)
        cert = extra_data['vs_cert']
        self.assertTrue(pkiutils.verifyPEMCertificate(cert))

    @patch("evotevotings.elections.utils.get_secret_entry", return_value=False)
    def test_invalid_private_key(self, mock_fn):
        """ Check for invalid certificates """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.user_id)
        self.assertFalse(extra_data)

    @patch("evotevotings.elections.utils.pkiutils.signObject",
           return_value=(1, None))
    def test_unsinged_objects(self, mock_fn):
        """ Check for unsigned object """
        extra_data = get_extra_election_data(self.election.pk,
                                             self.voter.user_id)
        self.assertFalse(extra_data)


class StepTwoTest(ValidCertificateTestCase,
                  ServiceTestUtils, ValidatorServiceTestUtils):
    """ Class to test the services workflow Step 2"""

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

        # Validate signature (test_validate_signature)

        # Validate Chain certificate (test_system_certificate)

        # Validate Voting System certificate
        # (test_valid_voting_system_certificate)

        # Validate Root certificate (test_valid_root_certificate)

        # Validate Election period (test_election_period)

        # Validate ElectionVoter permission to vote
        # (test_voter_permission)

        # Validate Voting Status (test_voter_status)

        # Sign the blind vote (test_blind_signature_signin)

    def test_system_certificate(self):
        """ Check if voting system certificate was issued
            with chain root certificate """
        election_id = self.election.id
        certs = get_certificates(election_id)
        vote_system_cert = certs['votesys']
        root_cert = certs['root']
        chain = pkiutils.verifyCertificateChain(
            vote_system_cert,
            root_cert
        )

        self.assertTrue(chain)

    def test_valid_root_certificate(self):
        """ Check if root certificate is within validation period """
        root_cert = self.certificates['root']
        not_expired = not_expired_certificate(root_cert)
        self.assertTrue(not_expired)

    def test_valid_voting_system_certificate(self):
        """ Check if certificate is within validation period """
        vote_system_cert = self.certificates['votesys']
        not_expired = not_expired_certificate(vote_system_cert)
        self.assertTrue(not_expired)

    def test_election_period(self):
        """ Check if election is in voting period """
        self.assertTrue(self.election.is_within_voting_period)

    def test_voter_permission(self):
        """ Check if election voter can vote in this election """
        self.assertIs(self.voter.election, self.election)

    def test_voter_status(self):
        """ Check if vote is available for the voter in the election"""
        self.assertTrue(self.voter.is_vote_available[1])

    def test_validate_signature(self):
        """ Check if the user + election signature is valid"""
        signature = self.election_data['combo_id']
        votesys_cert = self.certificates['votesys']
        (error_code, obj) = pkiutils.verifyObjectSignature(
            signature,
            votesys_cert
        )
        self.assertIsNone(error_code)
